;
; tealib.inc
; Bumblebee <bbbee@mailcity.com>
;
; void TEAIt(const unsigned char *inbuff, unsigned char *outbuff,
;  const unsigned long *const key, const unsigned long bufflen,
;  const unsigned long operation)
;
; operation must be 0 to cipher (other to decipher).
; key must point to a 128 bits key
;
; bufflen must be 64 bits aligned.
;
;
TEAIt   proc
        push    ebp
        mov     ebp,esp
        pushad        
        mov     esi,dword ptr [ebp+8]
        mov     edi,dword ptr [ebp+12]
        mov     ecx,dword ptr [ebp+20]
        mov     edx,dword ptr [ebp+24]
        mov     ebx,8
@TEAIt1:
        push    dword ptr [ebp+16]
        push    edi
        push    esi

        or      edx,edx
        jnz     @TEAItOp1
        call    TEABlkEncipher
        jmp     @TEAItOp2
@TEAItOp1:
        call    TEABlkDecipher
@TEAItOp2:
        add     esi,ebx
        add     edi,ebx

        sub     ecx,8
        jecxz   @TEAIt2
        jmp     @TEAIt1
@TEAIt2:
        popad
        mov     esp,ebp
        pop     ebp
        retn    20
TEAIt   endp

;
; void TEABlkEncipher(const unsigned long *const v,unsigned long *const w,
;  const unsigned long * const k)
;
; asmified and barely optimized.
;
TEABlkEncipher  proc
        push    ebp
        mov     ebp,esp
        sub     esp,20
        pushad
        mov     ebx,dword ptr [ebp+16]
        mov     edx,dword ptr [ebp+8]

        xor     ecx,ecx
        mov     eax,dword ptr [edx]
        mov     edx,dword ptr [edx+4]
        mov     dword ptr [ebp-4],9E3779B9h
        mov     esi,dword ptr [ebx]
        mov     dword ptr [ebp-8],esi
        mov     esi,dword ptr [ebx+4]
        mov     dword ptr [ebp-12],esi
        mov     esi,dword ptr [ebx+8]
        mov     dword ptr [ebp-16],esi
        mov     ebx,dword ptr [ebx+12]
        mov     dword ptr [ebp-20],ebx
        mov     ebx,32
        jmp     @blkenc2
@blkenc1:
        add     ecx,dword ptr [ebp-4]
        mov     esi,edx
        shl     esi,4
        add     esi,dword ptr [ebp-8]
        lea     edi,dword ptr [ecx+edx]
        xor     esi,edi
        mov     edi,edx
        shr     edi,5
        add     edi,dword ptr [ebp-12]
        xor     esi,edi
        add     eax,esi
        mov     esi,eax
        shl     esi,4
        add     esi,dword ptr [ebp-16]
        lea     edi,dword ptr [ecx+eax]
        xor     esi,edi
        mov     edi,eax
        shr     edi,5
        add     edi,dword ptr [ebp-20]
        xor     esi,edi
        add     edx,esi
@blkenc2:
        mov     esi,ebx
        dec     ebx
        or      esi,esi
        jnz     @blkenc1
        mov     ecx,dword ptr [ebp+12]
        mov     dword ptr [ecx],eax
        mov     dword ptr [ecx+4],edx

        popad
        mov     esp,ebp
        pop     ebp
        retn    12
TEABlkEncipher  endp

;
; void TEABlkDecipher(const unsigned long *const v,unsigned long *const w,
;   const unsigned long * const k)
;
; asmified and barely optimized.
;
TEABlkDecipher  proc
        push    ebp
        mov     ebp,esp
        sub     esp,20
        pushad
        mov     ebx,dword ptr [ebp+16]
        mov     eax,dword ptr [ebp+8]

        mov     edx,dword ptr [eax]
        mov     eax,dword ptr [eax+4]
        mov     ecx,0C6EF3720h
        mov     dword ptr [ebp-4],9E3779B9h
        mov     esi,dword ptr [ebx]
        mov     dword ptr [ebp-8],esi
        mov     esi,dword ptr [ebx+4]
        mov     dword ptr [ebp-12],esi
        mov     esi,dword ptr [ebx+8]
        mov     dword ptr [ebp-16],esi
        mov     ebx,dword ptr [ebx+12]
        mov     dword ptr [ebp-20],ebx
        mov     ebx,32
        jmp     @blkdec2
@blkdec1:
        mov     esi,edx
        shl     esi,4
        add     esi,dword ptr [ebp-16]
        lea     edi,dword ptr [ecx+edx]
        xor     esi,edi
        mov     edi,edx
        shr     edi,5
        add     edi,dword ptr [ebp-20]
        xor     esi,edi
        sub     eax,esi
        mov     esi,eax
        shl     esi,4
        add     esi,dword ptr [ebp-8]
        lea     edi,dword ptr [ecx+eax]
        xor     esi,edi
        mov     edi,eax
        shr     edi,5
        add     edi,dword ptr [ebp-12]
        xor     esi,edi
        sub     edx,esi
        sub     ecx,dword ptr [ebp-4]
@blkdec2:
        mov     esi,ebx
        dec     ebx
        or      esi,esi
        jnz     @blkdec1
        mov     ecx,dword ptr [ebp+12]
        mov     dword ptr [ecx],edx
        mov     dword ptr [ecx+4],eax

        popad
        mov     esp,ebp
        pop     ebp
        retn    12
TEABlkDecipher  endp
