;
; text.asm
; Bumblebee <bbbee@mailcity.com>
;
; Description:
;
;  1. Calc crc32+size of the plugin and the HASH of the DLL
;  2. Use it as part of the 128 bits key for TEA cipher
;  3. Encrypts the 128 bits key using RSA cipher and append to the plugin
;  4. Dumps base64 encoded
;
; IMPORTANT NOTICE: if RSA key is n bits long, that n is not stored, so
; you won't able to know how many bits used :D Moreover even TEA key is
; 128 bits long it will grow to fit RSA size when stored. Doesn't matter.
; Just try to get the DLL, crc32 is there to verify all goes fine.
; HASH of the DLL is just to check this signature if for that module, in
; other way a hacker should force his plugin to fit crc32 and the size
; to use this signature to distribute a fake module.
;
; Module will look like (before base64 encoding):
;
;         encrypted with RSA -> TEA KEY         n bytes
;                    TEA KEY =  CRC32           4 bytes
;                               DLL SIZE        4 bytes
;                               HASH of DLL     256 bits
;         encrypted with TEA -> DLL             m bytes
;
;
.486p
locals
.model flat,STDCALL

        extrn           ExitProcess:PROC
        extrn          LoadLibraryA:PROC
        extrn        GetProcAddress:PROC
        extrn           FreeLibrary:PROC
        extrn           CreateFileA:PROC
        extrn             WriteFile:PROC
        extrn              ReadFile:PROC
        extrn           GetFileSize:PROC
        extrn           CloseHandle:PROC

LINESIZE        equ     60

.DATA

; plugin name to load
plugindll       db      'plugin.dll',0
; plugin output
plugintxt       db      'plugin.txt',0

; tea key (change it each time but respect the size)
                align   4
                        ; TEA key is 16 chars long... but needs the
                        ; space for the RSA calcs
teaKey          db      '7845392846374526'
                dd      (BN_DWORD-4) dup (0)
teaCiphered     dd      BN_DWORD dup (0)

bytes           dd      0
fHnd            dd      0
dllSize         dd      0
encodedSize     dd      0
endln           db      0dh,0ah
buffer2ptr      dd      0

; change those values
buffer0         dd      5*1024 dup (0)          ; to store dll plain
buffer1         dd      5*1024 dup (0)          ; to store dll encrypted
buffer2         dd      6*4*1024 dup (0)        ; to store base64 enc dll

.CODE
inicio:
                                                ; 1st get the dll
        xor     eax,eax
        push    eax
        push    00000080h
        push    3
        push    eax
        push    00000001h
        push    80000000h
        push    offset plugindll
        call    CreateFileA
        cmp     eax,-1
        je      failedToOpen

        mov     dword ptr [fHnd],eax

        push    0h
        push    eax
        call    GetFileSize
        mov     [dllSize],eax

        ; read de DLL
        xor     eax,eax
        push    eax
        push    offset bytes
        push    dword ptr [dllSize]
        push    offset buffer0
        mov     ecx,[dllSize]
        lea     eax,teaKey
        mov     [eax+4],ecx                     ; store size
        push    dword ptr [fHnd]
        call    ReadFile

        push    dword ptr [fHnd]
        call    CloseHandle

        ; calc crc32 of the dll
        push    dword ptr [dllSize]
        push    offset buffer0
        call    CRC32

        mov     dword ptr [teaKey],eax          ; store CRC32

        lea     esi,teaKey                      ; calc the hash
        add     esi,8                           ; for the dll
        push    esi
        push    dword ptr [dllSize]
        push    offset buffer0
        call    hash

        ; encrypt the dll. TEA algorithm needs
        ; blocks of 64 bits, so we need to align the data.
        ; probably 1-6 bytes will be added at the end, no matter
        ; coz we store the DLL size to avoid those problems. You'll
        ; get later the exact DLL you paked :)
        ;
        ; Remember to skip BN_BYTE bytes at the begining of the buffer
        ; to store later the TEA key encrypted with RSA.
        ;
        push    0
        mov     eax,[dllSize]
        mov     ecx,8
        xor     edx,edx
        div     ecx
        or      edx,edx
        jz      greatItIsAligned
        inc     eax
greatItIsAligned:
        mul     ecx                             ; force 64 bits aligment
        mov     [dllSize],eax
        push    eax
        push    offset teaKey
        lea     esi,buffer1
        add     esi,BN_BYTE
        push    esi
        push    offset buffer0
        call    TEAIt                           ; TEA

        ; encrypt TEA key with RSA. Coz RSA uses n bits key, and
        ; i'm sure you'll use n>128, TEA key will grow to fit RSA
        ; size. So a 1024 bits RSA key will make the TEA 128 bits
        ; key use 1024 bits in disk.
        push    offset teaKey
        push    offset teaCiphered
        push    offset rsa_e
        push    offset rsa_n
        call    rsa_main                        ; RSA TEA key

        lea     esi,teaCiphered
        lea     edi,buffer1
        mov     ecx,BN_BYTE
        rep     movsb                           ; store TEA ciphered

        ; each 3 bytes will remain 4. So you can calc:
        ;
        ; ((TEA+CRC32+SIZE+DLL)/3)*4
        ;
        ; Bah, this will be done at home. I don't expect you
        ; encode modules itw :D
        ;
        lea     eax,buffer1
        lea     edx,buffer2
        mov     ecx,[dllSize]
        add     ecx,BN_BYTE
        call    encodeBase64                    ; encode into Base64

        mov     dword ptr [encodedSize],ecx

        xor     eax,eax                         ; save it to disk
        push    eax
        push    00000020h
        push    2
        push    eax
        push    00000001h OR 00000002h
        push    40000000h
        lea     eax,plugintxt
        push    eax
        call    CreateFileA
        cmp     eax,-1
        je      failedToOpen

        mov     dword ptr [fHnd],eax

        lea     eax,buffer2
        mov     [buffer2ptr],eax
        sub     dword ptr [encodedSize],LINESIZE
saveLoop:
        push    0
        push    offset bytes
        push    LINESIZE
        push    dword ptr [buffer2ptr]
        push    dword ptr [fHnd]
        call    WriteFile

        push    0
        push    offset bytes
        push    2
        push    offset endln
        push    dword ptr [fHnd]
        call    WriteFile

        add     dword ptr [buffer2ptr],LINESIZE

        cmp     dword ptr [encodedSize],LINESIZE
        jbe     endWrite
        sub     dword ptr [encodedSize],LINESIZE
        jmp     saveLoop

endWrite:
        push    0
        push    offset bytes
        push    dword ptr [encodedSize]
        push    dword ptr [buffer2ptr]
        push    dword ptr [fHnd]
        call    WriteFile

        push    0
        push    offset bytes
        push    2
        push    offset endln
        push    dword ptr [fHnd]
        call    WriteFile

        push    dword ptr [fHnd]
        call    CloseHandle

failedToOpen:
        push    0h
        call    ExitProcess

include test.inc
include ../inc/rsalib.inc
include ../inc/tealib.inc
include ../inc/crc32lib.inc
include ../inc/base64.inc
include ../inc/hash.inc

Ends
End     inicio


